--[[
    FuBar_MagicMarker - MagicMarker FuBar plugin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--]]

local mod  = AceLibrary("AceAddon-2.0"):new("FuBarPlugin-2.0", "AceDB-2.0")
local L = LibStub("AceLocale-3.0"):GetLocale("MagicMarker", false)
local C = LibStub("AceConfigDialog-3.0")
local T = AceLibrary("Tablet-2.0")
local DBOpt = LibStub("AceDBOptions-3.0")

local MM = MagicMarker
local MMDB = MagicMarkerDB

mod:RegisterDB("MMFuDB")
mod:RegisterDefaults('profile', { hideTitle = false })

mod.hasIcon = true
mod.canHideText = true
mod.canHideIcon = true
mod.blizzardTooltip = false
mod.cannotDetachTooltip = true
mod.showHintWhenDetached = false


local tinsert = tinsert
local sort = table.sort

local db

local options = {
   type = "group",
   args = {
      config = { 
	 type = "toggle",
	 name =  L["Toggle config dialog"],
	 desc = L["Toggle the Magic Marker configuration dialog."],
	 set = "ToggleConfigDialog",
	 get = function() return C.OpenFrames["Magic Marker"] ~= nil end,
	 handler = MM,
	 order = 1
      }, 
      toggle = {
	 type = "toggle", 
	 name = L["Toggle event handling"],
	 desc = L["Enable or disable the event handling, i.e whether or not Magic Marker will insert mobs into the mob database, mark mobs etc."],
	 set = "ToggleMagicMarker",
	 get = function() return MM.addonEnabled end,
	 handler = MM,
	 order = 2
      },
      assignments = {
	 type = "execute",
	 name = L["Report raid assignments"],
	 desc = L["Report the raid icon assignments to raid/party chat"]..".",
	 func = "ReportRaidMarks",
	 handler = MM
      },
      reset = {
	 type = "execute",
	 name = L["Reset raid icon cache"],
	 desc = L["Reset the raid icon cache. Note that this honors the Magic Marker options such as preserve raid marks."],
	 func = "ResetMarkData",
	 handler = MM,
      },
      commsettings = {
	 type = "group",
	 name = L["Data Sharing"],
	 desc = L["Data Sharing"],
	 order = 1200,
	 args = {}
      },
      settings = { 
	 type = "group",
	 name = L["General Options"],
	 desc = L["General Options"],
	 order = 1100,
	 args = {}
      },
      profiledata = {
	 type = "group",
	 name = "Profile",
	 desc = "Profile",
	 order = 1300,
	 args = {}
      },
      hideTitle = {
	 type = "toggle",
	 name = L["Short FuBar Text"],
	 desc = L["Hide Magic Marker from the FuBar status text."],
	 set = function() db.hideTitle = not db.hideTitle mod:Update() end,
	 get = function() return db.hideTitle end,
      },

      cache = {
	 type = "group",
	 name = L["Raid mark layout caching"],
	 desc = L["RAIDMARKCACHEHELP"],
	 order = 1000,
	 handler = MM,
	 args = {
	    save = {
	       type = "execute",
	       name = L["Save party/raid mark layout"],
	       desc = L["Save the current raid mark layout."],
	       func = "CacheRaidMarks",
	    },
	    load = {
	       type = "execute",
	       name = L["Load party/raid mark layout"],
	       desc = L["Load the currently saved raid mark layout."],
	       func = "MarkRaidFromCache",
	    },
	 }
      },
      tmpl = {
	 type = "group",
	 name = L["Raid group target templates"],
	 desc  = L["RAIDTMPLHELP"],
	 handler = MM,
	 order = 900
      },
   }
}
mod.OnMenuRequest = options

function mod:OnProfileEnable()
   db = self.db.profile 
end

local function capitalize(str)
   return strupper(str.sub(str,1,1))..str.sub(str, 2)
end

local function CopyAce3Config(src, dst, handler)
   local tmpdata
   for option, data in pairs(src) do
      if data.type ~= "header" and data.type ~= "description" then
	 local isnum, validate 
	 tmpdata = dst[option] or {}
	 -- Need to copy the table since we're modifying it
	 if data.type == "select" then
	    local values = data.values
	    validate = {}
	    if handler and type(data.values) == "string" then
	       values = handler[data.values](handler, data)
	    end
	    for key, val in pairs(values) do
	       if type(key) == "number" then
		  isnum = true
	       end
	       validate[tostring(key)] = val
	    end	    
	 end
	 if next(tmpdata) then
	    tmpdata.validate = validate
	 else
	    for key,val in pairs(data) do
	       if key ~= "values" then
		  tmpdata[key] = val
	       end
	    end
	    if not tmpdata.desc then tmpdata.desc = tmpdata.name end
	    local var = {option}
	    if tmpdata.type == "input" then
	       tmpdata.type = "text"
	       tmpdata.usage = L["Profile name"]		  
	    elseif tmpdata.type == "select" then	    
	       tmpdata.type = "text"
	       tmpdata.validate = validate
	    end
	    
	    if not handler then
	       tmpdata.get = function() local val = MM:GetProfileParam(var) if isnum then return tostring(val) else return val end end
	       tmpdata.set = function(val) if isnum then val = tonumber(val) end MM:SetProfileParam(var,val) MM:NotifyChange() end
	    else
	       if tmpdata.get then
		  tmpdata.get = function()
				   local val = handler[data.get](handler, var) if isnum then return tostring(val) else return val end end
	       end
	       if tmpdata.set then
		  if data.confirm then
		     tmpdata.set = function(val)
				      if isnum then val = tonumber(val) end mod:ConfirmAction(data.confirmText, val, handler, data.set) end
		  else
		     tmpdata.set = function(val)
				      if isnum then val = tonumber(val) end handler[data.set](handler, var,val) MM:NotifyChange() end
		  end
	       end
	    end
	 end
	 dst[option] = tmpdata
      end      
   end
end

function mod:OnInitialize()
   db = self.db.profile
   if not options.args.tmpl.args then
      args = {}
      for cmd,data in pairs(MM.MarkTemplates) do
	 if cmd ~= "arch" then
	    args[cmd] = {
	       type = "execute",
	       name = string.format("%s: %s", capitalize(cmd), data.desc),
	       desc = data.desc,
	       func = function() MM:MarkRaidFromTemplate(cmd) end,
	    }
	 end
      end
      options.args.tmpl.args = args
   end
   if MM.RegisterMMFu then
      MM:RegisterMMFu(mod)
   else
      MMFu = mod
   end
end

function mod:ConfirmAction(text, value, handler, method) 
   if not StaticPopupDialogs["MMFU_CONFIRM_DIALOG"] then
      StaticPopupDialogs["MMFU_CONFIRM_DIALOG"] = {}
   end
   local t = StaticPopupDialogs["MMFU_CONFIRM_DIALOG"]
   t.text = text
   t.button1 = L["Yes"]
   t.button2 = L["No"]
   
   t.OnAccept = function()
		   handler[method](handler, nil, value)
		   MM:NotifyChange() 
		end
   t.timeout = 0
   t.whileDead = 1
   t.hideOnEscape = 1
   
   StaticPopup_Show("MMFU_CONFIRM_DIALOG")
end

   

function mod:OnEnable()
   local mmopts = MM:GetOptions()
   CopyAce3Config(mmopts.args.options.args.commsettings.args, options.args.commsettings.args)
   CopyAce3Config(mmopts.args.options.args.settings.args, options.args.settings.args)
   self:GenerateProfileConfig()
end

function mod:GenerateProfileConfig()
   local profile = DBOpt:GetOptionsTable(MM.db, true);
   options.args.profiledata.args = {}   
   CopyAce3Config(profile.args, options.args.profiledata.args, profile.handler)

   options.args.profiledata.name = profile.name
   options.args.profiledata.desc = profile.desc
   options.args.profiledata.handler = profile.handler
   mod:Update()
end

local function white(str)
   return string.format("|cffffffff%s|r", str)
end

local function yellow(str)
   return string.format("|cffffff00%s|r", str)
end

local markedTargets, totalTargets
function mod:SetTargetCount(marked, total)
   markedTargets = marked
   totalTargets = total
   self:OnTextUpdate()
end
function mod:OnTextUpdate()
   local str
   if db.hideTitle then
      str = white((MM.addonEnabled and L["Enabled"]) or L["Disabled"])
   else
      str = string.format("%s: |cffffffff%s|r", L["Magic Marker"],
			  (MM.addonEnabled and L["Enabled"]) or L["Disabled"])
   end
   if totalTargets and totalTargets > 0 then
      str = string.format("%s (%d/%d)", str, markedTargets, totalTargets)
   end
   self:SetText(str)
end

function mod:OnClick(button)
   if button == "LeftButton" then
      if IsAltKeyDown() then
	 MM:ResetMarkData(IsShiftKeyDown())
      elseif IsShiftKeyDown() then
	 MM:ToggleMagicMarker()
      else
	 MM:ToggleConfigDialog()
      end
   elseif button == "MiddleButton" then
      MM:ReportRaidMarks()
   end   
end

function mod:OnMouseUp(button)
   if button == "MiddleButton" then
      MM:ReportRaidMarks()
   end
end

function mod:OnTooltipUpdate()
   local zone = GetRealZoneText()
   local supercat = T:AddCategory('columns', 2)
   
   supercat:AddLine( 'text', L["Status"],
	       'text2', white((MM.addonEnabled and L["Enabled"]) or L["Disabled"]))
   supercat:AddLine( 'text', L["Zone"],'text2', white(zone))

   local zoneID, _, heroic = MM:GetZoneName(zone)
   local inInstance, type = IsInInstance()
   if inInstance then
      local difficulty = GetInstanceDifficulty()
      if type == "raid" then
	 local dl = L["Raid"];
	 if difficulty == 1 then dl = dl .. " - 10"
	 elseif difficulty == 2 then dl = dl .. " - 25"
	 elseif difficulty == 3 then dl = dl .. " - 10 ".. L["Heroic"]
	 elseif difficulty == 4 then dl = dl .. " - 25 ".. L["Heroic"]
	 end
	 supercat:AddLine( 'text', L["Difficulty"], 'text2', dl)
      elseif type == 'pvp' or type == 'arena' then
	 supercat:AddLine( 'text', L["Difficulty"], 'text2', "PvP")
      else
	 supercat:AddLine( 'text', L["Difficulty"],
			  'text2', white((heroic and L["Heroic"]) or L["Normal"]))
      end
   end
   local cat
   if MMDB.mobdata[zoneID] then
      cat = T:AddCategory(nil):AddLine('text', MM:GetZoneInfo(MMDB.mobdata[zoneID]))
   end
   
   local sortData = {}
   local hasData
   local cacheData = MM:GetMarkData()
   local valueToId = {}
   for id, data in pairs(cacheData) do
      if data.value then
	 local key = data.value * 10000 + id
	 valueToId[key] = id
	 tinsert(sortData,key )
	 hasData = true
      end
   end

   totalTargets  = 0
   markedTargets = 0

   if hasData then
      cat = T:AddCategory('text', 'Raid Mark Cache',  'columns', 4)
      sort(sortData, function(a,b) return a > b end)   
      cat:AddLine('text', yellow(L["Unit Name"]),
		  'text2', yellow(L["Mark Type"]), 
		  'text3', yellow(L["Score"]),
		  'text4', yellow(L["Marked"]),
		  'size', 10, 'size2', 10, 'size3', 10, 
		  'justify2', 'CENTER',
		  'justify3', 'RIGHT')
      local data
      for _, id in pairs(sortData) do
	 id = valueToId[id]
	 data = cacheData[id]
	 if data.uid then
	    local valid
	    totalTargets = totalTargets + 1
	    if data.valid == nil then
	       valid = "N/A"
	       markedTargets = markedTargets + 1
	    elseif data.valid then
	       markedTargets = markedTargets + 1
	       valid =  L["Yes"]
	    else
	       valid = L["No"]
	    end
	    unitData = MM:GetUnitHash(data.uid)
	    cat:AddLine('text', unitData and unitData.name or data.uid,
			'text2', MM:GetCCName(data.ccid, data.value),
			'text3', string.format("%2.2f", data.value),
			'text4', valid,
			'size', 10, 'size2', 10, 'size3', 10, 
			'justify2', 'CENTER',
			'justify3', 'RIGHT',
			'hasCheck', true,
			'checkIcon', MM:GetIconTexture(id),
			'checked', true
		     )
	 end
      end
   end
   self:OnTextUpdate()
   T:SetHint(L["TOOLTIP_HINT"])
end

